﻿using System.Globalization;
using System.Xml;

namespace työ;

/// @author ainimatikainen
/// @version 12.3.2025
/// 
/// <summary>
/// Ohjelma laskee tuulen hyytävyyden lämpötilan, tuulennopeuden ja kosteuden perusteella.
/// Se hakee tarvittavat säätiedot Ilmatieteenlaitoksen avoimen datan rajapinnasta.
/// </summary>
class TuulenHyytavyys
{
    /// <summary>
    /// Ohjelma, joka hakee säätiedot ja laskee tuulen hyytävyyden.
    /// </summary>
    static async Task Main()
    {
        string[] timeStamps = {
            "2024-03-25T12:00:00Z", 
            "2024-03-26T12:00:00Z", 
            "2024-03-27T12:00:00Z", 
            "2024-03-28T12:00:00Z", 
            "2024-03-29T12:00:00Z", 
            "2024-03-30T12:00:00Z", 
            "2024-03-31T12:00:00Z"  
        };

        string place = "Jyväskylä"; 
        string baseUrl = "https://opendata.fmi.fi/wfs?service=WFS&version=2.0.0&request=GetFeature" +
                         "&storedquery_id=fmi::observations::weather::timevaluepair" +
                         $"&place={Uri.EscapeDataString(place)}" +
                         "&parameters=t2m,rh,windspeedms"; 
        
        try
        {
            using (HttpClient client = new HttpClient())
            {
                for (int i = 0; i < timeStamps.Length; i++) 
                {
                    string url = baseUrl + $"&starttime={timeStamps[i]}&endtime={timeStamps[i]}"; 

                    HttpResponseMessage response = await client.GetAsync(url);
                    response.EnsureSuccessStatusCode(); 
                    string xmlContent = await response.Content.ReadAsStringAsync();

                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.LoadXml(xmlContent);
                    
                    XmlNamespaceManager nsManager = new XmlNamespaceManager(xmlDoc.NameTable);
                    nsManager.AddNamespace("wml2", "http://www.opengis.net/waterml/2.0");

                    
                    XmlNode? temperatureNode = xmlDoc.SelectSingleNode("(//wml2:MeasurementTVP/wml2:value)[1]", nsManager);
                    XmlNode? humidityNode = xmlDoc.SelectSingleNode("(//wml2:MeasurementTVP/wml2:value)[2]", nsManager);
                    XmlNode? windSpeedNode = xmlDoc.SelectSingleNode("(//wml2:MeasurementTVP/wml2:value)[3]", nsManager);

                    
                    string temperatureStr = temperatureNode?.InnerText.Trim() ?? "Ei löydy";
                    string humidityStr = humidityNode?.InnerText.Trim() ?? "Ei löydy";
                    string windSpeedStr = windSpeedNode?.InnerText.Trim() ?? "Ei löydy";

                    Console.WriteLine($"\nAjankohta: ({timeStamps[i]}):");
                    Console.WriteLine($"Jyväskylän lämpötila: {temperatureStr} °C");
                    Console.WriteLine($"Ilmankosteus: {humidityStr} %");
                    Console.WriteLine($"Tuulen nopeus: {windSpeedStr} m/s");

                   
                    if (double.TryParse(temperatureStr, NumberStyles.Any, CultureInfo.InvariantCulture, out double temperature) &&
                        double.TryParse(humidityStr, NumberStyles.Any, CultureInfo.InvariantCulture, out double humidity) &&
                        double.TryParse(windSpeedStr, NumberStyles.Any, CultureInfo.InvariantCulture, out double windSpeed))
                    {
                        double windChill = TuulenHyytavyysLasku(temperature, windSpeed, humidity);
                        Console.WriteLine($"Tuulen hyytävyys: {windChill:F2} °C");
                    }
                    else
                    {
                        Console.WriteLine("Säädatan muuntaminen numeroiksi ei onnistu"); 
                    }
                }
            }
        }
        catch (HttpRequestException e) 
        {
            Console.WriteLine("HTTP-pyyntö epäonnistui: " + e.Message);
        }
        catch (Exception ex)
        {
            Console.WriteLine("Virhe: " + ex.Message);
        }
    }
    
    

    /// <summary>
    /// ohjelma askee tuulen hyytävyyden käyttäen lämpötilaa, tuulennopeutta ja kosteutta
    /// </summary>
    /// <param name="lampotila">Ilman lämpötila (°C)</param>
    /// <param name="nopeus">Tuulennopeus (m/s)</param>
    /// <param name="kosteus">Suhteellinen ilmankosteus (%)</param>
    /// <returns>Tuulen hyytävyyden indeksi (°C)</returns>
       
    private static double TuulenHyytavyysLasku(double lampotila, double nopeus, double kosteus)
    {
        double TuulenHyytavyysLasku = 13.12 + 0.6215 * lampotila - 11.37 * Math.Pow(nopeus, 0.16) + 0.3965 * Math.Pow(lampotila, 0.16) + (0.05 * kosteus);
        
        return TuulenHyytavyysLasku;
    }
    
}